// (C) Copyright 2013 Hewlett-Packard Development Company, L.P.
/**
 * @type (GroupDeployView)
 * This is a view class for Group Deploy View.
 */
define(['hpsum/presenter/group/GroupPresenter',
    'hpsum/presenter/node/NodePresenter',
	'hp/core/Router',
	'hp/core/UrlFragment',
    'hp/core/Notifications',
	'hp/core/HelpMenu',
	'hp/view/DialogView',
	'hp/core/Localizer',
	'hp/view/FormStateView',
	'hp/model/Session',
    'jquery',
    'hpsum/view/node/jquery.hpsumInstallables',	
    'hpsum/view/node/jquery.hpsumWarnings',	
    'hp/lib/jquery.hpStackedPanels',
    'lib/jquery.selectBox',
    'lib/jquery.dataTables',
    'lib/jquery.dataTables.rowReordering',
    'hp/lib/jquery.hpCollapsible',
    'hp/lib/jquery.hpToggle',	
	'hp/lib/jquery.hpSearchCombo'
	],

function(presenter, nodePresenter, router, urlFragment, notifications, helpMenu, DialogView, localizer, FormStateView, session) {"use strict";

	var GroupDeployView = ( function() {

		//Consts can go here
		var DEPLOY = '#hpsum-group-deploy-ok',
			CANCEL = '#hpsum-group-deploy-close',
			DEPLOY_CONTENTS = '#hpsum-group-deploy',
			GROUP_NODE_DEPLOY_CONTENT = '#hpsum-group-node-deploy-data',
			PANEL_SELECTOR = '#hpsum-deploy-panel-selector',
			FIRST_ANCHOR = PANEL_SELECTOR + ' a[href]:first',
			FORM = '#hpsum-group-deploy-add-form',
			PANELS = '#hpsum-group-deploy-panels',
			NAME = '#hpsum-group-deploy-name',
			REBOOTOPTION = '#hpsum-deploy-reboot-option',
			INSTALLATIONOPTION = '#hpsum-installation-option',
			DELAYSEC = '#hpsum-group-deploy-delay-seconds',
			DELAYSECERR = '#hpsum-group-deploy-delay-seconds-err',
			MESSAGE = '#hpsum-group-deploy-reboot-message',
			ROUTE_PREFIX = '/group/deploy',
			COMPVERBOSEOPTION = '#hpsum-comp-verbose-checkbox',
			NONECHECKBOX = '.nonecheckbox',
			USESNMPCHECKBOX = '.useSNMPcheckbox',
			USEAGENTLESSCHECKBOX = '.useAgentlesscheckbox',
			USEWEBEMCHECKBOX = '.useWEBEMcheckbox',
			DOWNGRADECHKBOXCOMP = '.downgradechkbox',
			REWRITECHKBOXCOMP = '.rewritechkbox',
			FIRMWARECHKBOXCOMP = '.firmwarechkbox',
			SOFTWARECHKBOXCOMP = '.softwarechkbox',
			SPLASH_DIALOG = '#hpsum-group-analysis-dialog',
			SPLASH_SPINNER = '#hpsum-group-analysis-spinner',
			THROBBER = '#hpsum-group-deploy-throbber',
			CURRENT_DATETIME = '#hpsum-group-current-datetime',
			INSTALL_ERROR='#force-options-tbody-err',
			REBOOT_DISABLE = '.hpsum-otu-group-deploy-disable',
			REBOOT_DELAY_SEC = '60',
			NODE_DIALOG = '#hpsum-group-node-dialog',
			NODE_COMP = '#hpsum-node-components',
			NODE_ANALYSIS = "#hpsum-node-deploy-analysis";
				
			/**
			 * Constructor
			 */
			function GroupDeployView() {
				var formStateView = new FormStateView();
				var groupItem = null,
					setAttributesOnGroup = false,
					analysisWaitDialog = null, 

					ForceOptions = { 
						hapi: {					
							installation_option:{
								downgrade: false,
								rewrite:false,
								force:false,
								firmware :false,
								software :false						
							},					
						}
					},
					defaultOptiondetails = { 
						hapi: {					
							reboot_options : {
								action : 'Never',
								delay : REBOOT_DELAY_SEC,
								message : 'Reboot at user request after deploy'
							},					
						}
					},

					optiondetails = defaultOptiondetails,
					initialGetattr = null,
					NodeDialogContents = null,
					groupNodeDetails = [],
					warningmsg = '',
					failedDependencyCount = 0, 
					SelectedComponents = 0,
					failedDependencyFlag = false,
					applyOptions = false,
					nodeIdList = [],
					getUpdatelistnodeCount = 0,
					group_id = "",
					cancelDeploy = false,
					progressTimer,
					NodeDialog = null,
					isWarningsChecked = false,
					allNodeWarningsSelected = false;
					

				function onItemChange(item) {
					groupItem = item.hapi;

					if (groupItem) {
						cancelDeploy = false;
						if(group_id != groupItem.group_id){
							group_id = groupItem.group_id;
							$(NAME).text(groupItem.group_name);
							enableDeployBtn(true);
							//Get the group attributes and display on UI
							loadAttributes();
							if(groupItem.group_state.match(/UAN/i) && (groupItem.group_phase.match(/UserActionNeeded|NoComponents/i))){
								populateDeployData();
							}
							else{
								cancelDeploy = true;
							}
						}
					} else {
						cancelDeploy = true;
					}
					if(cancelDeploy){
						$(CANCEL).trigger('click');
					}
				}
				
				function loadAttributes()
				{
					presenter.getAttributes(groupItem, {
						success : function(data) {
							if(data.hapi.hcode == 0)
								showoptions(data);
						},
						error : function(data) {
						}
					});
				}
				
				//Read the Option details like reboot options, verbose logging before deployment
				function loadOptionDetails() {
					var atleastOneOptionSelected = false;

					optiondetails.hapi.reboot_options.action = $(REBOOTOPTION).val();
					
					if (($(DELAYSEC).val() != '') && ($(DELAYSEC).val().match(/^[0-9]+$/) !== null)) {
						optiondetails.hapi.reboot_options.delay = $(DELAYSEC).val();
					}
					
					if ($(MESSAGE).val() != '') {
						optiondetails.hapi.reboot_options.message = $(MESSAGE).val();
					}
					if($(COMPVERBOSEOPTION).is(':checked'))
					{
						optiondetails.hapi.verbose = true;
						atleastOneOptionSelected = true;
					}
					if(initialGetattr.hapi.reboot_options.action != optiondetails.hapi.reboot_options.action ||
						initialGetattr.hapi.reboot_options.delay != optiondetails.hapi.reboot_options.delay ||
						initialGetattr.hapi.reboot_options.message != optiondetails.hapi.reboot_options.message) {
						atleastOneOptionSelected = true;
					}
					
					if (!atleastOneOptionSelected) {
						optiondetails.hapi.reboot_options = {};
					} else {
						setAttributesOnGroup = true;
					}
					if(!allNodeWarningsSelected && isWarningsChecked)
					{
						optiondetails.hapi.ignore_warnings = true;
						setAttributesOnGroup = true;
					}
					atleastOneOptionSelected = false;
				}
				
				//Frame the setattributes for installation options based on type selected
				function  getInstallationOptions()
				{
					var type = $(INSTALLATIONOPTION).val();
					ForceOptions.hapi.installation_option = {};
								 
					ForceOptions.hapi.installation_option.downgrade = false;
					ForceOptions.hapi.installation_option.rewrite   = false;
					ForceOptions.hapi.installation_option.software = false;
					ForceOptions.hapi.installation_option.firmware = false;
					ForceOptions.hapi.installation_option.forcesoftware = false;
					ForceOptions.hapi.installation_option.forcefirmware = false;
					
					if ( type == "upgradeBoth"){
						ForceOptions.hapi.installation_option.firmware = true;
						ForceOptions.hapi.installation_option.software = true;
					}
					else if ( type == "upgradeFirmware"){
						ForceOptions.hapi.installation_option.firmware = true;
					}
					else if ( type == "upgradeSoftware"){
						ForceOptions.hapi.installation_option.software = true;
					}					
					else if ( type == "downgradeFirmware"){
						ForceOptions.hapi.installation_option.downgrade = true;
						ForceOptions.hapi.installation_option.firmware = true;
						ForceOptions.hapi.installation_option.forcefirmware = true;
					}
					else if ( type == "downgradeSoftware"){
						ForceOptions.hapi.installation_option.downgrade = true;
						ForceOptions.hapi.installation_option.software = true;
						ForceOptions.hapi.installation_option.forcesoftware = true;
					}
					else if ( type == "downgradeBoth"){
						ForceOptions.hapi.installation_option.downgrade = true;
						ForceOptions.hapi.installation_option.firmware = true;
						ForceOptions.hapi.installation_option.forcefirmware = true;
						ForceOptions.hapi.installation_option.software = true;
						ForceOptions.hapi.installation_option.forcesoftware = true;
					}
					else if ( type == "rewriteFirmware"){
						ForceOptions.hapi.installation_option.rewrite = true;
						ForceOptions.hapi.installation_option.firmware = true;
						ForceOptions.hapi.installation_option.forcefirmware = true;
					}
					else if ( type == "rewriteSoftware"){
						ForceOptions.hapi.installation_option.rewrite = true;
						ForceOptions.hapi.installation_option.software = true;
						ForceOptions.hapi.installation_option.forcesoftware = true;
					}
					else if ( type == "rewriteBoth"){
						ForceOptions.hapi.installation_option.rewrite = true;
						ForceOptions.hapi.installation_option.firmware = true;
						ForceOptions.hapi.installation_option.forcefirmware = true;
						ForceOptions.hapi.installation_option.software = true;
						ForceOptions.hapi.installation_option.forcesoftware = true;
					}
					else if ( type == "firmware"){
						ForceOptions.hapi.installation_option.downgrade = true;
						ForceOptions.hapi.installation_option.rewrite = true;
						ForceOptions.hapi.installation_option.firmware = true;
						ForceOptions.hapi.installation_option.forcefirmware = true;
					}
					else if ( type == "software"){
						ForceOptions.hapi.installation_option.downgrade = true;
						ForceOptions.hapi.installation_option.rewrite = true;
						ForceOptions.hapi.installation_option.software = true;
						ForceOptions.hapi.installation_option.forcesoftware = true;
					}
					else if ( type == "both"){
						ForceOptions.hapi.installation_option.downgrade = true;
						ForceOptions.hapi.installation_option.rewrite = true;
						ForceOptions.hapi.installation_option.firmware = true;
						ForceOptions.hapi.installation_option.forcefirmware = true;
						ForceOptions.hapi.installation_option.software = true;
						ForceOptions.hapi.installation_option.forcesoftware = true;
					}
					if(initialGetattr.hapi.installation_option.downgrade != ForceOptions.hapi.installation_option.downgrade ||
						initialGetattr.hapi.installation_option.rewrite != ForceOptions.hapi.installation_option.rewrite ||
						initialGetattr.hapi.installation_option.software != ForceOptions.hapi.installation_option.software ||
						initialGetattr.hapi.installation_option.firmware != ForceOptions.hapi.installation_option.firmware) {
						applyOptions = true;
					}
				}
				
				//Read the installation options and send setattributes to engine
				function applyInstallationOptions()
				{
					//Load the installation options
					getInstallationOptions();
					if(applyOptions){
						showAnalysisWaitDialog(true);
						formStateView.setMessage({
							summary: localizer.getString("hpsum.node.installationOptions.applyingOptions"),
							status: 'ok',
							changing: true,
						});
						presenter.setAttributes(groupItem,ForceOptions, {
							success : function(data){
								getGroupStatus();
							},
							error : onSetattributeError
						});
					}
				}

				//Get the group status and populate the node group data
				function getGroupStatus()
				{
					var timer = setTimeout(function() {
						presenter.getGroupStatus({group_id: groupItem.group_id}, {
							success : function(data) {
								if (data.hapi.hcode == 0) {
									if (data.hapi.group_status.match(/Ready to start deploy/)) {
										clearTimeout(timer);	
										timer = '';
										//Get the Latest Node data on the UI
										populateDeployData();
										closeAnalysisWaitDialog();									
										if(applyOptions)
										{
											formStateView.setMessage({
												summary: localizer.getString("hpsum.node.installationOptions.optionsSuccess"),
												status: 'ok',
												changing: false,
											});
											applyOptions = false;
											loadAttributes();
										}
									}
									else
									{
										getGroupStatus();
									}
								}
							},
							error : function(data) {
								clearTimeout(timer);
								timer = '';
								closeAnalysisWaitDialog();
							}
						});
					}, 10000); 
				}
				
				function showoptions(data)
				{
					initialGetattr = null;
					if(data && data.hasOwnProperty("hapi"))
					{
						initialGetattr = data;
						if(data.hapi.hasOwnProperty("ignore_warnings") && data.hapi.ignore_warnings){
							isWarningsChecked = true;
							allNodeWarningsSelected = true;
						}
						if(data.hapi.hasOwnProperty("verbose") && data.hapi.verbose){
							$(COMPVERBOSEOPTION).attr('checked', true);
						}
						if(data.hapi.hasOwnProperty("reboot_options") && data.hapi.reboot_options)
						{
							$('#hpsum-deploy-reboot-option option').filter(function() {
							if(new RegExp(this.value, 'ig').test( data.hapi.reboot_options.action))
		    		            $(this).attr('selected', true);
							});
							if( -1 < data.hapi.reboot_options.delay){
								$(DELAYSEC).val(data.hapi.reboot_options.delay);				
							}
							else{
								$(DELAYSEC).val(REBOOT_DELAY_SEC);				
							}
						
							$(MESSAGE).attr('value', data.hapi.reboot_options.message);
							$(REBOOTOPTION).selectBox('reset');
						}
					
						if(data.hapi.hasOwnProperty("installation_option") && data.hapi.installation_option)
						{
							var rewriteOption = data.hapi.installation_option.rewrite;
							var downgradeOption = data.hapi.installation_option.downgrade;
							var firmwareOption = data.hapi.installation_option.firmware;
							var softwareOption = data.hapi.installation_option.software;
							
							if ( !rewriteOption  && !downgradeOption )
							{
								if(firmwareOption && softwareOption){
									$(INSTALLATIONOPTION).val('upgradeBoth');
								}
								else if(firmwareOption){
									$(INSTALLATIONOPTION).val('upgradeFirmware');
								}
								else if(softwareOption){
									$(INSTALLATIONOPTION).val('upgradeSoftware');
								}
							}
							else if ( rewriteOption  && downgradeOption )
							{
								if(firmwareOption && softwareOption)
									$(INSTALLATIONOPTION).val('both');
								else if(firmwareOption)
									$(INSTALLATIONOPTION).val('firmware');
								else if(softwareOption)
									$(INSTALLATIONOPTION).val('software');
							}
							else if ( rewriteOption )
							{
								if(firmwareOption && softwareOption)
									$(INSTALLATIONOPTION).val('rewriteBoth');
								else if(firmwareOption)
									$(INSTALLATIONOPTION).val('rewriteFirmware');
								else if(softwareOption)
									$(INSTALLATIONOPTION).val('rewriteSoftware');
							}
							else if ( downgradeOption )
							{
								if(firmwareOption && softwareOption)
									$(INSTALLATIONOPTION).val('downgradeBoth');
								else if(firmwareOption)
									$(INSTALLATIONOPTION).val('downgradeFirmware');
								else if(softwareOption)
									$(INSTALLATIONOPTION).val('downgradeSoftware');
							}
											
							$(INSTALLATIONOPTION).selectBox('reset');										
						}						
					}
				}

				function errormsg()
				{
					$(INSTALL_ERROR).hide();
				}


				function onDeploy() {
					if(validateRebootDelay()){
						//Disable the deploy button 
						enableDeployBtn(false);
						
						loadOptionDetails();
						//Check the reboot options and verbose log and send setattributes
						if (setAttributesOnGroup) {
							presenter.setAttributes(groupItem, optiondetails, {
								success : onSetattributeSuccess,
								error : onSetattributeError
							});
						}
						else{
							//Continue with deploy
							onDeployContinue();
						}
					}
				}

				function onSetattributeSuccess(item) {
					setAttributesOnGroup = false;
					//Continue with deploy
					onDeployContinue();
				}

				function onSetattributeError(errorMessage) {
					setAttributesOnGroup = false;
					notifications.show(errorMessage, 'error');
					enableDeployBtn(false)
					$(CANCEL).trigger('click');
				}
			
				
				/**
				 * @private
				 * Continue the deploy procedure from the Analysis onwards 
				 */
				function onDeployContinue() {
				
					$(THROBBER).show();
										
					presenter.deploy(groupItem, {
						success : onDeploySuccess,
						error : onDeployError
					});
				}

				function onDeploySuccess(item) {
					$(THROBBER).hide();
					notifications.show(groupItem.group_name + ' Group ' + localizer.getString("hpsum.group.deployStarted"), 'ok');
					$(CANCEL).trigger('click');
				}

				function onDeployError(errorMessage) {
					$(THROBBER).hide();
					notifications.show(errorMessage, 'error');
					$(CANCEL).trigger('click');
				}

				function enableDeployBtn(able) {
					$(DEPLOY).attr('disabled_org', able? 0:1);
					$('.ignorewarnings').each(function() {
						if (!$(this).is(':checked')) {
							able = false;
						}
					});
					if (able) {
						$(DEPLOY).removeAttr("disabled");
					} else {
						$(DEPLOY).attr('disabled', 'disabled');
					}
				}

				function changeLocation(location) {
					if (location.length > ROUTE_PREFIX.length) {
						$(PANELS).hpStackedPanels('setPanelByLocation', '#' + location);
					} else {
						$(PANELS).hpStackedPanels('setFirstPanel');
					}
				}

				function populateOptions() 
				{
					optiondetails = defaultOptiondetails;
					$(NONECHECKBOX).attr('checked', false);
					$(USESNMPCHECKBOX).attr('checked', false);
					$(USEAGENTLESSCHECKBOX).attr('checked', false);
					$(USEWEBEMCHECKBOX).attr('checked', false);
					$(INSTALLATIONOPTION).val('upgradeBoth');
					$(INSTALLATIONOPTION).selectBox('reset');
					$(REBOOTOPTION).selectBox('value', 'Never');
					$(INSTALL_ERROR).hide();
					$(DELAYSEC).val(REBOOT_DELAY_SEC);
					$('textarea' + MESSAGE).attr('value', 'Reboot at user request after deploy');
				}
			
				function initGrpNodesTable(grouptableid) 
				{
					var oTab = $(grouptableid).dataTable({
						oLanguage: {
							sEmptyTable: localizer.getString('hpsum.group.noNodes'),
							sSearch: localizer.getString('hpsum.common.search')
								   },
						bPaginate : false,
						bProcessing : true,
						bFilter : true,
						bSort : true,
						bAutoWidth : false,
						bInfo : false,
						bRetrieve : true,
						bDestroy : true,
						bServerSide:false,
						aoColumns : [
							{mDataProp: 'node_ip'},
							{mDataProp: 'status',
							fnRender: function (oObj) {
								if(oObj.aData.failedDepCount ){
									return '<div class="hp-status"><div class="hp-status-value tooltip hp-status-error" tooltip="Dependency Errors"><span class="hp-value">error</span></div></div>';
								}
								else if(oObj.aData.selectedCompCount > 0){
									return '<div class="hp-status"><div class="hp-status-value tooltip hp-status-warning" tooltip="Node Update required"><span class="hp-value">warning</span></div></div>';
								}
								else if(oObj.aData.totalCompCount == 0){
									return '<div class="hp-status"><div class="hp-status-value tooltip hp-status-ok" tooltip="No Applicable components"><span class="hp-value">ok</span></div></div>';
								}
								else if(oObj.aData.selectedCompCount == 0){
									return '<div class="hp-status"><div class="hp-status-value tooltip hp-status-ok" tooltip="No Updates applicable"><span class="hp-value">ok</span></div></div>';
								}
								
							}, sWidth: 5},
							{mDataProp: 'link' ,
							fnRender: function (oObj) {
								if(oObj.aData.state.match(/UAN/i))
								{
									return '<a id = "componentLink" href="javascript:void(0);">' + localizer.getString('hpsum.node.applicableComponents') + '</a>';
								}
							}},
							{mDataProp: 'node_status'},
							{mDataProp: 'node_id', "bVisible": false, sWidth: 5},
							{mDataProp: 'state', "bVisible": false, sWidth: 5},
							{mDataProp: 'type', "bVisible":false, sWidth:5},
							{mDataProp: 'selectedCompCount', "bVisible":false, sWidth:5},
							{mDataProp: 'totalCompCount', "bVisible":false, sWidth:5},
							{mDataProp: 'failedDepCount', "bVisible":false, sWidth:5}
						],
						aaSorting: [[ 0, "asc" ]],				    
						fnRowCallback : function( nRow, aData, iDisplayIndex, iDisplayIndexFull ) {
							return nRow;
						}
						});			
				}

				//Get the Node details and show in tabular format
				function populateDeployData(){
					presenter.getUpdateList(groupItem, {
						success : function(data) {
							groupNodeDetails = [];
							if (data.hasOwnProperty('hapi') && data.hapi.output_data.hasOwnProperty('nodes')) {
								var nodeData = data.hapi.output_data.nodes.node;
								for (var idx = 0; idx < nodeData.length; idx++){
									if(nodeData[idx].node_state === "UAN"){
										groupNodeDetails.push(nodeData[idx]);
									}
								}
								populateGroupNodeData();
							}
						},
						error : function(data) {
						}
					});
				}
			
				//Display the node data in tabular format
				function populateGroupNodeData() {
					var htmlWarnings = '',
						failedDependencyFlag = false,
						selectedCompCount = 0;
						
					for (var idx = 0; idx < groupNodeDetails.length; idx++) {
						htmlWarnings += '<div><label><b>' + groupNodeDetails[idx].ip + '</b></label><div id="hpsum-group-warnings-' + groupNodeDetails[idx].node_id + '"></div></div><br />';
						$('#hpsum-group-node-cname-' + groupNodeDetails[idx].node_id).html(groupNodeDetails[idx].ip);
					}
					$(GROUP_NODE_DEPLOY_CONTENT, DEPLOY_CONTENTS).empty();
					
					if (htmlWarnings.length > 0) {
						htmlWarnings += '<div><label><input class="useWEBEMcheckbox ignorewarnings-tbd" type="checkbox" />' + localizer.getString('hpsum.common.ignoreWarnings') + '</label></div>';
					}
					htmlWarnings = '<div id="hpsum-group-warnings-messages">' + htmlWarnings + '</div><div id="hpsum-group-no-warnings"><label>' + localizer.getString('hpsum.common.noWarnings') + '</label></div>'
					$('#hpsum-group-warnings').html(htmlWarnings);
					$('#hpsum-group-warnings-messages').hide();
					$('#hpsum-group-no-warnings').show();
					
					for (var idx = 0; idx < groupNodeDetails.length; idx++) {
						$("#hpsum-group-nodename-" + groupNodeDetails[idx].node_id).html(groupNodeDetails[idx].ip);
						$("#hpsum-group-warnings-" + groupNodeDetails[idx].node_id).hpsumWarnings(groupNodeDetails[idx], false);
						if (groupNodeDetails[idx].warnings.length > 0) {
							if ($('.ignorewarnings-tbd')[0]) {
								$('.ignorewarnings-tbd').addClass('ignorewarnings').removeClass('ignorewarnings-tbd');
								if(isWarningsChecked){
									$('.ignorewarnings').attr("checked", true);
								}
								$('#hpsum-group-warnings-messages').show();
								$('#hpsum-group-no-warnings').hide();
								$('.ignorewarnings').on('change', function(){
									if ($(this).is(':checked')) {
										isWarningsChecked = true;
										$(DEPLOY).attr("disabled", $(DEPLOY).attr("disabled_org") == 1);
									} else {
										isWarningsChecked = false;
										$(DEPLOY).attr("disabled", true);
									}
									presenter.setAttributes(groupItem,{
										hapi : {
											ignore_warnings : isWarningsChecked,
										}
										}, {
										   success : function(){},
										   error : function() {}
									});
								});
								$('.ignorewarnings').attr("nid", groupNodeDetails[idx].node_id);
							} else {
								$('.ignorewarnings').attr("nid", $('.ignorewarnings').attr("nid") + " " + groupNodeDetails[idx].node_id);
							}
						}
						$('#hpsum-warnings-collapsible').removeClass('hp-collapsed');
					}
					$('.my-collapsible', DEPLOY_CONTENTS).hpCollapsible();
					
					$(GROUP_NODE_DEPLOY_CONTENT).append('<table class="hp-selectable" id="hpsum-group-deploy-table"><thead><tr><td><b>' 
											+ localizer.getString('hpsum.node.ip')+ '</b></td><td><b>' 
											+ localizer.getString("hpsum.oneTouchUpdate.statusStr") + '</b></td><td><b>' 
											+ localizer.getString('hpsum.node.componentDetails') + '</b></td><td><b>' 
											+ localizer.getString('hpsum.common.statusStr') + '</b></td><td></td><td></td></tr></thead></table>');
							   
					var grouptableid = '#hpsum-group-deploy-table';
					var groupNodeTable;
					
					//Initialize the Node Group Data table
					initGrpNodesTable(grouptableid);
					groupNodeTable = $(grouptableid).dataTable();
					
					//Clear the Group table;
					if(groupNodeTable){
						groupNodeTable.fnClearTable();
					}
					
					//Add the nodes to the Group table
					for(var idx = 0; idx < groupNodeDetails.length; idx++){
						if(groupNodeDetails[idx].has_failed_dependency > 0){
							failedDependencyFlag = true;
						}
						if(!groupNodeDetails[idx].ignore_warnings){
							allNodeWarningsSelected = false;
						}
						selectedCompCount += groupNodeDetails[idx].selected_comp_count;
						var nodeGroupItem = [{
							'node_ip': groupNodeDetails[idx].ip,
							'status': "",
							'link': "",
							'node_status': groupNodeDetails[idx].node_status,
							'node_id': groupNodeDetails[idx].node_id,
							'state': groupNodeDetails[idx].node_state,
							'type': groupNodeDetails[idx].type,
							'selectedCompCount': groupNodeDetails[idx].selected_comp_count,
							'totalCompCount': groupNodeDetails[idx].total_comp_count,
							'failedDepCount': groupNodeDetails[idx].has_failed_dependency
						}];
						
						groupNodeTable.fnAddData(nodeGroupItem);
					}
					if(failedDependencyFlag || selectedCompCount == 0){
						enableDeployBtn(false);
					}else{if(selectedCompCount > 0)
						enableDeployBtn(true);
					}
					
					$('.tooltip').hpTooltip();
					
					//Click event for the Applicable components
					$(grouptableid + " tbody tr").click( function( e ) {
						var srcElem;
						var data = $(grouptableid).dataTable().fnGetData( this );
						
						if(data != null)
						{
							//Check if firefox or (chrome and IE)
							if(e.srcElement)
								srcElem = e.srcElement;//Chrome and IE
							else
							{
								srcElem = e.target;//Firefox
							}
						
							if ( srcElem.tagName == 'A' ){
								showNodeCompDialog(data.node_id,data.type);
							}
						}
					});

				}
				
				//Show the dialog with the component information for that node
				function showNodeCompDialog(node_id,type) 
				{
					var node_uri = "/Session/" + session.getToken() + "/node/" + node_id;
					InitNodeDialog(node_id,type);
					$(NODE_ANALYSIS).attr("disabled", true);
					$(NODE_COMP).empty();
					$(NODE_COMP).append("<div id='node-baseline' style='margin:10px;' ></div>");

					var nodeGrpItem = {
						'node_id':node_id,
						 'type': type
					};

					var options = {
						command: 'init',
						nodeItem: nodeGrpItem,
						onSelectionChanged: onNodeSelectionChanged,
						onPopulateInstallSet: onNodePopulateInstallSet
						
					};
						
					//Call plugin to show the component list ();
					$('#node-baseline').hpsumInstallables(options);
					
				}
			   
		    //Callback function for the selection changed event
			function onNodeSelectionChanged(id, value, componentList) {		
				var selectedCompsCount = componentList.component.length;
				var failedDepCount = componentList.failed_dependency;
				
				//Disable Save button when node has failed dependencies or no components selected
				if(selectedCompsCount < 1 || !value){
					$(NODE_ANALYSIS).attr("disabled", true);
				}
				else{
					$(NODE_ANALYSIS).attr("disabled", false);
				}
			}
			
			//Callback function for deploy function from plugin
			function onNodePopulateInstallSet(node_id)
			{
				
			}
			
			function InitNodeDialog(nodeid,type)
			{
			    if(!NodeDialog) {
			     	var closefunc =  function(elem) {
						$('.hpsumInstallables').hpsumInstallables('destroy');
                        NodeDialog = null;
                     };
                    NodeDialog = new DialogView();
                    NodeDialog.init({contents: NodeDialogContents,
                        position : "center",
                        ok : function(){ closefunc() },
                        cancel: function(){ closefunc() }
                    }); 
                    //Register the click event for the SAVE button 
                    $(NODE_ANALYSIS, NodeDialogContents).off('click').on('click', function(ev) {
                        btnNodeSaveClicked.call(this,nodeid,type);
                    });
                }
			}
			
			function btnNodeSaveClicked(nodeid,type) 
			{
				//Show the spinning wheel for the anlaysis 
				showAnalysisWaitDialog();

				//Get the component list for the selected node
				var inst = $('#node-baseline').data('plugin_hpsumInstallables');
				var componentsList = inst.getComponents();
				
				var nodeItem = {
					'node_id':nodeid
				};
				
				nodePresenter.analysis(nodeItem, componentsList, {
					node_id: nodeItem.node_id,
					success : function(data) {
						analyzeSuccess(data, this.node_id);
					},
					error : function(errorInfo) {
						analyzeError(errorInfo);
					}
				});
					
				//start the timer to check the node status
				progressTimer = setInterval(function () {
					var nodeItem = {
						'node_id':nodeid
					};
					nodePresenter.getNodeStatus(nodeItem, {
						nodeItem: nodeItem,
						success : function(data) {
							nodeStatusSuccess(data, nodeItem);
						},
						error : function(errorInfo) {
							
						}
					});
				}, 3000);
			}
			
			function nodeStatusSuccess(data, nodeItem)
			{
				if(data.hapi.node_status.node_state === "ANALYSIS")
				{
					
				}
				else if(data.hapi.node_status.node_state === "UAN")
				{
					//Re populate all the nodes information to get the latest data
					populateDeployData();
					clearInterval(progressTimer);
					closeAnalysisWaitDialog();
	
					//disable Save button
					$(NODE_ANALYSIS).attr("disabled", true);
					//Close the Node Component dialog
					NodeDialog.cancel();

				}
				else if(data.hapi.node_status.node_state === "ERROR")
				{
					clearInterval(progressTimer);
					closeAnalysisWaitDialog();
				}
			}
			
			function analyzeSuccess(data, nodeid) {
			
			}
			
			function analyzeError(error) {
				closeAnalysisWaitDialog();
			}
			
			function showAnalysisWaitDialog(flag) {
					analysisWaitDialog = $(SPLASH_DIALOG);
					analysisWaitDialog.dialog({
						modal : true,
						dialogClass : 'hp-simple-dialog',
						position : "center",
						draggable : false,
						closeOnEscape: false,
						resizable : false,
						width : 280,
						height : 180
					});
					if(flag)
						$('#hpsum-group-analysis-txt').html(localizer.getString("hpsum.group.applyingInstallationOptions"));
					else
						$('#hpsum-group-analysis-txt').html(localizer.getString("hpsum.group.applyingSelectionChanges"));
				$(SPLASH_SPINNER).show();
				analysisWaitDialog.dialog('open');
			}

			function closeAnalysisWaitDialog() {
				$(SPLASH_SPINNER).hide();
				analysisWaitDialog.dialog('destroy');
			}		
			
			function validateRebootDelay() {
				var rebootopt = $('#hpsum-deploy-reboot-option option:selected').val();
				var delaySec = $(DELAYSEC).val();
				var valid = true;
				
				if (rebootopt != "Never") {
					if((isNaN(delaySec) && !isFinite(delaySec)) || 
					  (delaySec < 60 || delaySec > 3600)) {
						$(DELAYSEC).addClass("hp-error");
						$(DELAYSECERR).text(localizer.getString("hpsum.node.rebootDelaySecondsErrMsg"));
						$(DELAYSECERR).show();
						valid = false;
					}
					else {
						$(DELAYSEC).val(parseInt(delaySec));
						$(DELAYSEC).removeClass("hp-error");
						$(DELAYSECERR).hide();
					}
				}
				return valid;
			}
			
			function reset() {
				populateOptions();
				group_id = "";
				nodeIdList = [];
				cancelDeploy = false;
				getUpdatelistnodeCount = 0;
				isWarningsChecked = false;
				allNodeWarningsSelected = false;
				$(DELAYSEC).attr("disabled", true);
				$(MESSAGE).attr("disabled", true);
				$(COMPVERBOSEOPTION).attr('checked', false);
				$(DELAYSEC).removeClass("hp-error");
				$(DELAYSECERR).hide();
				formStateView.reset();
			}
			
			
			/**
			 * @public
			 * Initialize Group Deploy View
			 */
			this.init = function() {

				$(DEPLOY).live('click', function(ev) {
					onDeploy.call(this);
					ev.preventDefault();
				});
				
				$(CANCEL).live('click', function() {
					$('.hpsumInstallables').hpsumInstallables('destroy');
					presenter.off("itemChange", onItemChange);
					router.go(urlFragment.replaceView(router.location(), 'show'));
				});
				
				$('.require-analysis').on('change', function(){
					if(failedDependencyCount <= 0 && SelectedComponents > 0){
						enableDeployBtn(true);
					}
					else
					{
						enableDeployBtn(false);
					}
				});
				
				//By Default reboot-options value is Never so disable the Reboot delay and Reboot message.
				$(REBOOT_DISABLE).find('input,textarea,select,button').prop("disabled", true);
				//$(ANALYSIS).attr('tooltip', localizer.getString('hpsum.common.deployPreviewToolTip')).hpTooltip();
				$(REBOOTOPTION).change(function() {
					var type = $('#hpsum-deploy-reboot-option option:selected').val();
					if (type == "Never") {
					$(REBOOT_DISABLE).find('input,textarea,select,button').prop("disabled", true);
					$(DELAYSEC).val('');
					$(DELAYSEC).removeClass("hp-error");
					$(DELAYSECERR).hide();
					}
					else
					{
					$(REBOOT_DISABLE).find('input,textarea,select,button').prop("disabled", false);
					}
				});
				$(REBOOTOPTION).selectBox({autoWidth: false});
				$(DELAYSEC).unbind('keyup').bind('keyup',function(){
					$(DELAYSECERR).hide();
					$(DELAYSEC).removeClass('hp-error');
					validateRebootDelay();
				}); 
				
				$(DELAYSEC).bind('change', function(){
					$(DELAYSECERR).hide();
					$(DELAYSEC).removeClass('hp-error');
				}); 
				
				$(PANELS).hpStackedPanels({
					panelSelector : PANEL_SELECTOR
				});

				$(".hp-collapsible").hpCollapsible();

				enableDeployBtn(true);
				
				
				$('#hpsum-downgrade-rewrite-text').html(localizer.getString("hpsum.node.installationOptions.message"));
				$('#hpsum-downgrade-rewrite-text').removeClass('hp-collapsed');
				$(INSTALL_ERROR).hide();
				$("#hpsum-installation-option").change(function() {
					applyInstallationOptions.call();
				});
				$("#hpsum-reboot-options-collapsible").css("margin-top","-20px");
				$(INSTALLATIONOPTION).selectBox({autoWidth: false});
				$(INSTALLATIONOPTION).val('None');
				$(INSTALLATIONOPTION).selectBox('reset');
				$('#hp-force-downgrade-rewrite').hide();
					
				router.watch('Group Deploy section', ROUTE_PREFIX + '.*', {
					parent : ROUTE_PREFIX,
					change : changeLocation
				});
				
				formStateView.init({form: FORM,
					routePattern: ROUTE_PREFIX + '.*',
					autoTrack: false
				});
				
				this.resume();
			};

				/**
				 * @public
				 * Resume Group Deploy View
				 */
				this.resume = function() {
					$(THROBBER).hide();
					presenter.on("itemChange", onItemChange);
					$(PANELS).hpStackedPanels('resume');
					reset();
					$('#hpsum-group-deploy').hpCollapsible('expand');
					NodeDialogContents = $(NODE_DIALOG).detach().show();
					$(NODE_COMP).empty();
					NodeDialog = null;
				};

				/**
				 * @public
				 * Pause Group Deploy View
				 */
				this.pause = function() {
					presenter.off("itemChange", onItemChange);
					$(PANELS).hpStackedPanels('pause');
				};
				
			}

		return GroupDeployView;
	}());

	return new GroupDeployView();
});
