#!/bin/sh
#set -x

# get current path which includes gatherlogs.sh 
#path=$(cd "dirname $0";pwd)
path=$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )
arch=$(`echo uname -m` | tr '[a-z]' '[A-Z]')

tablestodrop="CNodeData:sshkeyfile_location_h_STRING,sshkeyfile_passphrase_h_STRING,username_h_STRING,password_h_STRING,su_username_h_STRING,su_password_h_STRING,ass_username_h_STRING,ass_password_h_STRING,vcenter_username_h_STRING,vcenter_password_h_STRING \
CSessionSettings:proxy_username_h_STRING,proxy_password_h_STRING,icm_token_id_h_STRING \
CScalableUpdateGroup:username_h_STRING,password_h_STRING \
CMoonshotNodeData:switch_password_h_STRING,switch_enable_h_STRING,switchb_password_h_STRING,switchb_enable_h_STRING"

if [[ "$arch" = "X86_64" ]]; then
    arch="x64"
elif [[ "$arch" = "IA64" ]]; then
    arch="ia64"
else
    arch="x86"
fi

droptable="${path}/${arch}/hpsum_drop_table_${arch}"

datetime=`date +"%m-%d-%Y_%H-%M-%S"`
tarName=`echo "$path/HPSUM_Logs_${datetime}.tar"`
tmpFile=`echo "directory_listing.txt"`
hpsumDirListing="hpsum_directory_listing.txt"
syslogFile="syslog.log"
oldsyslogFile="OLDsyslog.log"
topOutput="topOutput.txt"
fsState="fsState.txt"
upTime="upTime.txt"

#determine which zip program to use, or none
gzip=`which gzip 2>/dev/null`
compress=`which compress 2>/dev/null`

if [ "x$gzip" != "x" ]; then
	zipper=$gzip
else
	if [ "x$compress" != "x" ]; then
		zipper=$compress
	else
		#no zip
		zipper="touch"
	fi
fi

echo "Gathering HPSUM logs..."
found=0

#listing of current dir and sub dir, usually dir running HPSUM, to capture files, dates/times
ls -Rl > "$tmpFile"


if [ ! -f $tmpFile ];then 
    tarName="/tmp/HPSUM_Logs_${datetime}.tar"
    tmpFile="/tmp/.directory_listing.txt"
    hpsumDirListing="/tmp/hpsum_directory_listing.txt"
    syslogFile="/tmp/syslog.log"
    oldsyslogFile="/tmp/OLDsyslog.log"
    topOutput="/tmp/topOutput.txt"
    fsState="/tmp/fsState.txt"
    upTime="/tmp/upTime.txt"
    ls -Rl > $tmpFile
fi

tar -cf "$tarName" "$tmpFile"
rm "$tmpFile"

#Gather system statistics
if [ ${arch} = "ia64" ]; then
    top -f ${topOutput}
    bdf > ${fsState}
else
    top -n 1 -b > ${topOutput}
    df > ${fsState}
fi
tar -rf "$tarName" "$topOutput"
tar -rf "$tarName" "$fsState"
uptime > ${upTime}
tar -rf "$tarName" "$upTime"
rm -f ${upTime} ${fsState} ${topOutput}

# firmware wrapper logs
if [ -d /var/log/hp-sum ]; then
  tar -rf "$tarName" /var/log/hp-sum/* 2>/dev/null
  found=1
fi

tmpHPSUM=""
if [ -f hpsum.ini ];then 
	tmpHPSUM=`grep -E "^temp_dir" hpsum.ini | awk -F= '{print $2"/HPSUM"}'`
fi
if [ "x$tmpHPSUM" = "x" ]; then
	tmpHPSUM=/tmp/HPSUM
	if [ -f $tmpHPSUM/hpsum.ini ];then 
		tmpHPSUM=`grep -E "^temp_dir" $tmpHPSUM/hpsum.ini | awk -F= '{print $2"/HPSUM"}'`
	fi
fi

#HPSUM directory listing 
ls -Rl "$tmpHPSUM" > "$hpsumDirListing"
echo "   " >> $hpsumDirListing
echo Available disk space: >> $hpsumDirListing
df -k >> "$hpsumDirListing"
tar -rf "$tarName" "$hpsumDirListing"
rm "$hpsumDirListing"

#debug logs
if [ -d $tmpHPSUM ]; then
  findCommand=`find $tmpHPSUM -name "*.txt" -o -name "*.log" -o -name "*.xml" -o -name "*.trace" -o -name "*.ini"`

  echo $findCommand | while read file; do
    tar -rf "$tarName" $file 2>/dev/null
  done

  # Collect hpsum.pdb and drop the tables only in normal mode
  if [ "$PHOENIX" != "1" ]; then
     if [ -f ${droptable} ]; then
        findCommand=`find $tmpHPSUM -name "*.pdb"`
        for dbfilename in $findCommand
        do
           #Copy the db file and drop the tables from the new db file
           dir_name=$(dirname ${dbfilename});
           file_name=$(basename ${dbfilename});
           modifiedfile="${dir_name}/mod_${file_name}"
           cp -p ${dbfilename} ${modifiedfile}

           # Drop the tables using drop_table and gather it
           result=`${droptable} ${modifiedfile} ${tablestodrop} 2>&1`
           # If drop_table utility fails for any reason, don't gather the DB file
           if [[ $? != 0 ]]; then
               echo -e "${result}. Skipping ${dbfilename} file.\n"
               rm -f ${modifiedfile} 2>/dev/null
               continue
           fi
           tar -rf "$tarName" ${modifiedfile} 2>/dev/null
           rm -f ${modifiedfile}
        done
     else
        echo "${droptable} is not present. Skipping DB files."
     fi
  fi

  found=1
fi

#user logs
if [ -d /var/hp/log ]; then
  tar -rf "$tarName" /var/hp/log/* 2>/dev/null
  found=1
fi

#component logs
if [ -d /var/cpq ]; then
  tar -rf "$tarName" /var/cpq/* 2>/dev/null
  found=1
fi

#system logs
if [ -f /var/adm/syslog/syslog.log ]; then
   tail -1000 /var/adm/syslog/syslog.log > "$syslogFile" 2>/dev/null
else
  if [ -f /var/log/messages ]; then 
     tail -1000 /var/log/messages > "$syslogFile" 2>/dev/null
  fi
fi
if [ -f "$syslogFile" ]; then
	tar -rf "$tarName" "$syslogFile"
	rm "$syslogFile"
fi

if [ -f /var/adm/syslog/OLDsyslog.log ]; then
   tail -1000 /var/adm/syslog/OLDsyslog.log > $oldsyslogFile 2>/dev/null
fi
if [ -f $oldsyslogFile ]; then
	tar -rf "$tarName" "$oldsyslogFile"
	rm "$oldsyslogFile"
fi

#hpsut logs
if [ -d /opt/hp/hpsut ]; then
  tar -rf "$tarName" /opt/hp/hpsut/hpsut*.log 2>/dev/null
  found=1
fi
#hpsutesxi logs
if [ -d /opt/hp/hpsutesxi ]; then
  findLogsCommand=`find /opt/hp/hpsutesxi -name "*.log" -o -name "*.pdb*" -o -name "HPSUM_Combined_Report_*.xml"`
  echo $findLogsCommand | while read file; do
    tar -rf "$tarName" $file 2>/dev/null
  done
  found=1
fi

#combined_report.xml
if [ -f "${path}"/HPSUM_Combined_Report_*.xml  ]; then
  tar -rf "$tarName" "${path}"/HPSUM_Combined_Report_*.xml 2>/dev/null
  found=1
fi

#zip if possible
if [ $found -eq 1 ]; then
  $zipper "$tarName"
  newName=`ls -1 "$tarName"*`
  echo "HPSUM logs are in $newName"
  exit 0
fi

rm "$tarName"
echo "No HPSUM logs found"
exit -1
